@@ -35,6 +35,12 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "./gridinit-utils.h"
 #include "./gridinit-internals.h"
 
+time_t supervisor_default_delay_KILL = SUPERVISOR_DEFAULT_TIMEOUT_KILL;
+
+static time_t _monotonic_seconds(void) {
+	return g_get_monotonic_time() / G_TIME_SPAN_SECOND;
+}
+
 /**
  * Temporary flag used by gridinit to mark services during a refresh.
  */
@@ -90,6 +96,7 @@ struct my_rlimits_s {
 
 struct child_s {
 	struct child_s *next;
+	time_t delay_before_KILL;
 	gchar *command;
 	pid_t pid;
 	uid_t uid;
@@ -99,15 +106,18 @@ struct child_s {
 	guint32 user_flags;
 	GSList *env;
 
-	gchar key[SUPERVISOR_LIMIT_CHILDKEYSIZE];
-	gchar group[2048];
-
 	/* Useful stats */
 	guint counter_started;
 	guint counter_died;
+
+	/* wall-clock time */
+	time_t last_start;
+
+	/* monotonic-clock time */
 	time_t last_start_attempt;
 	time_t first_kill_attempt;
 	time_t last_kill_attempt;
+
 	struct {
 		time_t t0;
 		time_t t1;
@@ -118,16 +128,12 @@ struct child_s {
 
 	/* Child's startup properties */
 	struct my_rlimits_s rlimits;
+
+	gchar key[SUPERVISOR_LIMIT_CHILDKEYSIZE];
+	gchar group[2048];
 };
 
-static struct child_s SRV_BEACON = {
-	NULL, NULL, 0, 0, 0,
-	NULL, 0, 0, NULL,
-	"", "",     /* keys */
-	0, 0, 0, 0, 0, /* birth/death stats */
-	{0,0,0,0,0} /* deaths */,
-	{0,0,0}     /* limits */
-};
+static struct child_s SRV_BEACON = {};
 
 static supervisor_postfork_f *supervisor_cb_postfork = NULL;
 static void *supervisor_cb_postfork_udata = NULL;
@@ -232,8 +238,7 @@ _child_get_info(struct child_s *c, struct child_info_s *ci)
 	ci->gid = c->gid;
 	ci->counter_started = c->counter_started;
 	ci->counter_died = c->counter_died;
-	ci->last_start_attempt = c->last_start_attempt;
-	ci->last_kill_attempt = c->last_kill_attempt;
+	ci->last_start_attempt = c->last_start;
 
 	ci->rlimits.core_size = i64tolong(c->rlimits.core_size);
 	ci->rlimits.stack_size = i64tolong(c->rlimits.stack_size);
@@ -313,7 +318,7 @@ my_clear_env(void)
 }
 
 /**
- * Must be called after the fork, from the child, just befoe the execve
+ * Must be called after the fork, from the child, just before the execve
  */
 static char **
 _child_build_env(struct child_s *sd)
@@ -327,7 +332,7 @@ _child_build_env(struct child_s *sd)
 
 	new_env = calloc(1 + g_slist_length(sd->env), sizeof(char*));
 
-	/* Run the configured environement */
+	/* Run the configured environment */
 	for (i=0, l=sd->env; l && l->next ;l=l->next->next) {
 		k = l->data;
 		v = l->next->data;
@@ -383,7 +388,7 @@ _child_start(struct child_s *sd, void *udata, supervisor_cb_f cb)
 	gint argc;
 	gchar **args;
 	struct my_rlimits_s saved_limits;
-	
+
 	if (!sd || !sd->command) {
 		errno = EINVAL;
 		return -1;
@@ -393,29 +398,30 @@ _child_start(struct child_s *sd, void *udata, supervisor_cb_f cb)
 		errno = EINVAL;
 		return -1;
 	}
-	
+
 	bzero(&saved_limits, sizeof(saved_limits));
 
-	sd->last_start_attempt = time(0);
+	sd->last_start_attempt = _monotonic_seconds();
+	sd->last_start = time(0);
 
 	_child_set_rlimits(&(sd->rlimits), &saved_limits);
 	sd->pid = fork();
 
 	switch (sd->pid) {
 
-	case -1: /*error*/
+	case -1: /* error */
 		errsav = errno;
 		g_strfreev(args);
 		errno = errsav;
 		return -1;
 
-	case 0: /*child*/
+	case 0: /* child */
 		setsid();
 		sd->pid = getpid();
 		if (supervisor_cb_postfork != NULL)
 			supervisor_cb_postfork(supervisor_cb_postfork_udata);
 		reset_sighandler();
-		
+
 		/* change the rights before changing the working directory */
 		if (getuid() == 0) {
 			setgid(sd->gid);
@@ -426,9 +432,9 @@ _child_start(struct child_s *sd, void *udata, supervisor_cb_f cb)
 
 		_child_exec(sd, argc, args);
 		exit(-1);
-		return 0;/*makes everybody happy*/
+		return 0; /* makes everybody happy */
 
-	default: /*father*/
+	default: /* father */
 
 		INFO("Starting service [%s] with pid %i", sd->key, sd->pid);
 
@@ -458,14 +464,15 @@ static void
 _child_stop(struct child_s *sd)
 {
 	if (sd->pid > 0) {
-		time_t now = time(0);
+		time_t now = _monotonic_seconds();
 		if (sd->first_kill_attempt == 0)
 			sd->first_kill_attempt = now;
-		if (sd->first_kill_attempt > 0 && (now - sd->first_kill_attempt > SUPERVISOR_DEFAULT_TIMEOUT_KILL)) {
+		if (sd->delay_before_KILL > 0
+				&& sd->first_kill_attempt > 0
+				&& (now - sd->first_kill_attempt) > sd->delay_before_KILL) {
 			DEBUG("Service [%s] did not exit after 60s, sending SIGKILL", sd->key);
 			kill(sd->pid, SIGKILL);
-		}
-		else {
+		} else {
 			DEBUG("Sending SIGTERM to service [%s] pid %i", sd->key, sd->pid);
 			kill(sd->pid, SIGTERM);
 		}
@@ -484,7 +491,7 @@ _child_notify_death(struct child_s *sd)
 	sd->deaths.t3 = sd->deaths.t2;
 	sd->deaths.t2 = sd->deaths.t1;
 	sd->deaths.t1 = sd->deaths.t0;
-	sd->deaths.t0 = time(0);
+	sd->deaths.t0 = _monotonic_seconds();
 
 	if (FLAG_HAS(sd, MASK_NEVER_BROKEN))
 		return;
@@ -505,7 +512,7 @@ _child_should_be_up(struct child_s *sd)
 static void
 _child_debug(struct child_s *sd, const gchar *tag)
 {
-	time_t now = time(0);
+	time_t now = _monotonic_seconds();
 	DEBUG("%s [%s] flags=%04X now=%ld deaths{%ld,%ld,%ld,%ld,%ld}",
 		tag, sd->key, sd->flags, now,
 		now - sd->deaths.t0,
@@ -518,12 +525,10 @@ _child_debug(struct child_s *sd, const gchar *tag)
 static gboolean
 _child_can_be_restarted(struct child_s *sd)
 {
-	time_t now;
-
 	if (!_child_should_be_up(sd))
 		return FALSE;
 
- 	if (!sd->last_start_attempt)
+	if (!sd->last_start_attempt)
 		return TRUE;
 
 	/* here : already been started */
@@ -531,11 +536,11 @@ _child_can_be_restarted(struct child_s *sd)
 		return FALSE;
 
 	/* here : restart allowed */
-	if (!FLAG_HAS(sd,MASK_DELAYED))	
+	if (!FLAG_HAS(sd,MASK_DELAYED))
 		return TRUE;
 
 	/* here : restart delayed if died too early */
-	now = time(0);
+	time_t now = _monotonic_seconds();
 
 	_child_debug(sd, "DEAD");
 
@@ -629,12 +634,6 @@ supervisor_children_start_enabled(void *udata, supervisor_cb_f cb)
 	return count;
 }
 
-guint
-supervisor_children_startall(void *udata, supervisor_cb_f cb)
-{
-	return supervisor_children_start_enabled(udata, cb);
-}
-
 guint
 supervisor_children_mark_obsolete(void)
 {
@@ -663,27 +662,7 @@ supervisor_children_disable_obsolete(void)
 			count ++;
 		}
 	}
-	
-	return count;
-}
 
-guint
-supervisor_children_kill_obsolete(void)
-{
-	guint count;
-	struct child_s *sd;
-
-	count = 0U;
-
-	FOREACH_CHILD(sd) {
-		if (FLAG_HAS(sd,MASK_OBSOLETE)) {
-			if (sd->pid > 0) {
-				_child_stop(sd);
-				count ++;
-			}
-		}
-	}
-	
 	return count;
 }
 
@@ -694,7 +673,7 @@ supervisor_children_catharsis(void *udata, supervisor_cb_f cb)
 	guint count;
 	struct child_s *sd;
 	struct child_info_s ci;
-	
+
 	count = 0;
 	while ((pid_dead = waitpid(-1, NULL, WNOHANG)) > 0) {
 		FOREACH_CHILD(sd) {
@@ -768,13 +747,11 @@ supervisor_children_fini(void)
 
 
 gboolean
-supervisor_children_register(const gchar *key, const gchar *cmd, GError **error)
+supervisor_children_register(const gchar *key, const gchar *cmd)
 {
 	struct child_s *sd = NULL;
 
-	(void) error;
-
-	/*check if the service is present*/
+	/* check if the service is present */
 	FOREACH_CHILD(sd) {
 		if (0 == g_ascii_strcasecmp(sd->key, key)) {
 
@@ -788,7 +765,7 @@ supervisor_children_register(const gchar *key, const gchar *cmd, GError **error)
 		}
 	}
 
-	/*Child not found, it will be created*/
+	/* Child not found, it will be created */
 	sd = g_try_malloc0(sizeof(struct child_s));
 	if (NULL == sd) {
 		errno = ENOMEM;
@@ -796,6 +773,7 @@ supervisor_children_register(const gchar *key, const gchar *cmd, GError **error)
 	}
 
 	g_strlcpy(sd->key, key, sizeof(sd->key)-1);
+	sd->delay_before_KILL = supervisor_default_delay_KILL;
 	sd->flags = MASK_STARTED|MASK_RESPAWN|MASK_DELAYED;
 	sd->working_directory = g_get_current_dir();
 	sd->command = g_strdup(cmd);
@@ -810,8 +788,8 @@ supervisor_children_register(const gchar *key, const gchar *cmd, GError **error)
 	(void) supervisor_limit_get(SUPERV_LIMIT_THREAD_STACK, &(sd->rlimits.stack_size));
 	(void) supervisor_limit_get(SUPERV_LIMIT_MAX_FILES,    &(sd->rlimits.nb_files));
 	(void) supervisor_limit_get(SUPERV_LIMIT_CORE_SIZE,    &(sd->rlimits.core_size));
-	
-	/*ring insertion*/
+
+	/* ring insertion */
 	sd->next = SRV_BEACON.next;
 	SRV_BEACON.next = sd;
 
@@ -850,7 +828,7 @@ supervisor_children_kill_disabled(void)
 	FOREACH_CHILD(sd) {
 		/* Stop child that needs to be restarted */
 		if (FLAG_HAS(sd,MASK_RESTART))
-			_child_set_flag(sd, MASK_STARTED, FALSE);	
+			_child_set_flag(sd, MASK_STARTED, FALSE);
 
 		if (!_child_should_be_up(sd)) {
 			if (sd->pid > 0) {
@@ -860,7 +838,7 @@ supervisor_children_kill_disabled(void)
 			}
 		}
 	}
-	
+
 	return count;
 }
 
@@ -889,7 +867,7 @@ supervisor_children_enable(const char *key, gboolean enable)
 		_child_set_flag(sd, MASK_BROKEN, FALSE);
 
 		/* We reset the 'last_start_attempt' field. This is necessary
-		 * to explicitely restart services confiured with the 'cry'
+		 * to explicitely restart services configured with the 'cry'
 		 * or 'exit' value for their 'on_die' parameter */
 		sd->last_start_attempt = 0;
 	}
@@ -943,7 +921,7 @@ supervisor_children_repair_all(void)
 			count ++;
 		}
 	}
-	
+
 	errno = 0;
 	return count;
 }
@@ -1166,6 +1144,22 @@ supervisor_children_set_ids(const gchar *key, gint32 uid, gint32 gid)
 	return 0;
 }
 
+int
+supervisor_children_set_delay_sigkill(const char *key, time_t delay)
+{
+	if (!key) {
+		errno = EINVAL;
+		return -1;
+	}
+	struct child_s *sd;
+	if (!(sd = supervisor_get_child(key))) {
+		errno = ENOENT;
+		return -1;
+	}
+	sd->delay_before_KILL = delay;
+	return 0;
+}
+
 void
 supervisor_set_callback_postfork(supervisor_postfork_f *cb, void *udata)
 {
