@@ -54,9 +54,9 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "./gridinit_alerts.h"
 #include "../lib/gridinit-internals.h"
 
-#define USERFLAG_ONDIE_EXIT                                          0x00000001
-#define USERFLAG_PROCESS_DIED                                        0x00000002
-#define USERFLAG_PROCESS_RESTARTED				     0x00000004
+#define USERFLAG_ONDIE_EXIT        0x00000001
+#define USERFLAG_PROCESS_DIED      0x00000002
+#define USERFLAG_PROCESS_RESTARTED 0x00000004
 
 #define BOOL(i) ((i)!=0)
 
@@ -97,7 +97,6 @@ static volatile int flag_help = 0;
 static volatile int flag_quiet = 0;
 static volatile int flag_daemon = 0;
 static volatile int flag_running = ~0;
-static volatile int flag_cfg_reload = 0;
 static volatile int flag_check_socket = 0;
 static volatile int flag_more_verbose = 0;
 
@@ -1011,29 +1010,22 @@ _make_empty_env (void)
 static GHashTable*
 _cfg_extract_parameters (GKeyFile *kf, const char *s, const char *p, GError **err)
 {
-	gchar **all_keys=NULL, **current_key=NULL;
 	gsize size=0;
+	gchar **all_keys = g_key_file_get_keys (kf, s, &size, err);
+	if (!all_keys)
+		return NULL;
 
 	GHashTable *ht = _make_empty_env ();
-
-	all_keys = g_key_file_get_keys (kf, s, &size, err);
-	if (!all_keys)
-		goto error;
-	for (current_key=all_keys; current_key && *current_key ;current_key++) {
-		if (g_str_has_prefix(*current_key, p)) {
-			gchar *value = g_key_file_get_value (kf, s, *current_key, err);
-			g_hash_table_insert (ht, g_strdup(*current_key + strlen(p)), value);
+	for (gchar **pk = all_keys; all_keys && *pk ;pk++) {
+		const char *key = *pk;
+		if (g_str_has_prefix(key, p)) {
+			gchar *value = g_key_file_get_value (kf, s, key, err);
+			g_hash_table_insert (ht, g_strdup(key + strlen(p)), value);
 		}
 	}
 
 	g_strfreev(all_keys);
 	return ht;
-error:
-	if (ht)
-		g_hash_table_destroy(ht);
-	if (all_keys)
-		g_strfreev(all_keys);
-	return NULL;
 }
 
 static gchar*
@@ -1182,11 +1174,12 @@ _cfg_section_service(GKeyFile *kf, const gchar *section, GError **err)
 	gchar *str_command, *str_enabled, *str_startatboot, *str_ondie,
 		*str_uid, *str_gid,
 		*str_limit_stack, *str_limit_core, *str_limit_fd,
-		*str_wd, *str_group;
+		*str_wd, *str_group, *str_delay_sigkill;
 	gint32 uid, gid;
 
 	uid = gid = -1;
 	str_key = strchr(section, '.') + 1;
+	str_delay_sigkill = __get_and_enlist(&gc, kf, section, CFG_KEY_DELAY_KILL);
 	str_command = __get_and_enlist(&gc, kf, section, "command");
 	str_enabled = __get_and_enlist(&gc, kf, section, "enabled");
 	str_ondie = __get_and_enlist(&gc, kf, section, "on_die");
@@ -1227,7 +1220,7 @@ _cfg_section_service(GKeyFile *kf, const gchar *section, GError **err)
 	 * being reloaded. */
 	already_exists = _service_exists(str_key);
 
-	if (!supervisor_children_register(str_key, str_command, err))
+	if (!supervisor_children_register(str_key, str_command))
 		goto label_exit;
 
 	/* Enables or not. This is a lock controlled by the configuration
@@ -1323,6 +1316,11 @@ _cfg_section_service(GKeyFile *kf, const gchar *section, GError **err)
 	if (str_group)
 		supervisor_children_set_group(str_key, str_group);
 
+	if (str_delay_sigkill) {
+		time_t delay = g_ascii_strtoll(str_delay_sigkill, NULL, 10);
+		supervisor_children_set_delay_sigkill(str_key, delay);
+	}
+
 	rc = TRUE;
 
 label_exit:
@@ -1396,6 +1394,7 @@ _cfg_section_default(GKeyFile *kf, const gchar *section, GError **err)
 	gint64 limit_thread_stack = 1024LL * 1024LL;
 	gint64 limit_core_size = -1LL;
 	gint64 limit_nb_files = 8192LL * 1024LL * 1024LL;
+	gint64 delay_sigkill = -1LL;
 	gchar **p_key, **keys;
 
 	keys = g_key_file_get_keys(kf, section, NULL, err);
@@ -1408,10 +1407,13 @@ _cfg_section_default(GKeyFile *kf, const gchar *section, GError **err)
 
 		str = g_key_file_get_string(kf, section, *p_key, NULL);
 
-		if (!g_ascii_strcasecmp(*p_key, CFG_KEY_INHERIT)) {
+		if (!g_ascii_strcasecmp(*p_key, CFG_KEY_DELAY_KILL)) {
+			delay_sigkill = g_ascii_strtoll(str, NULL, 10);
+		}
+		else if (!g_ascii_strcasecmp(*p_key, CFG_KEY_INHERIT)) {
 			inherit_env = g_ascii_strtoll(str, NULL, 10);
 		}
-		if (!g_ascii_strcasecmp(*p_key, CFG_KEY_LIMIT_CORESIZE)) {
+		else if (!g_ascii_strcasecmp(*p_key, CFG_KEY_LIMIT_CORESIZE)) {
 			limit_core_size = g_ascii_strtoll(str, NULL, 10) * 1024LL * 1024LL;
 		}
 		else if (!g_ascii_strcasecmp(*p_key, CFG_KEY_LIMIT_NBFILES)) {
@@ -1508,6 +1510,10 @@ _cfg_section_default(GKeyFile *kf, const gchar *section, GError **err)
 		config_subdir = g_strndup(buf_includes, sizeof(buf_includes));
 	}
 
+	if (delay_sigkill >= 0) {
+		supervisor_default_delay_KILL = delay_sigkill;
+	}
+
 	return TRUE;
 }
 
@@ -1590,13 +1596,13 @@ _cfg_reload(gboolean services_only, GError **err)
 			return 0;
 		}
 		int glob_rc;
-		glob_t subfiles_glob;
-
-		bzero(&subfiles_glob, sizeof(subfiles_glob));
+		glob_t subfiles_glob = {};
 
 		DEBUG("Loading services files matching [%s]", config_subdir);
 
-		glob_rc = glob(config_subdir, GLOB_NOSORT|GLOB_MARK, notify_error, &subfiles_glob);
+		glob_rc = glob(config_subdir,
+				GLOB_BRACE|GLOB_NOSORT|GLOB_MARK,
+				notify_error, &subfiles_glob);
 		if (glob_rc != 0) {
 			if (glob_rc == GLOB_NOMATCH)
 				NOTICE("Service file pattern matched no file!");
@@ -1604,24 +1610,18 @@ _cfg_reload(gboolean services_only, GError **err)
 				WARN("reconfigure : glob error : %s", strerror(errno));
 		}
 		else {
-			char **p_str;
-
-			DEBUG("reconfigure : glob done, %"G_GSIZE_FORMAT" elements found", subfiles_glob.gl_pathc);
-			for (p_str=subfiles_glob.gl_pathv; p_str && *p_str ;p_str++) {
+			for (char **ps=subfiles_glob.gl_pathv; subfiles_glob.gl_pathv && *ps ;ps++) {
+				const char *path = *ps;
 				GError *gerr_local = NULL;
-				GKeyFile *sub_kf = NULL;
-
-				TRACE("Loading a new file");
-
-				sub_kf = g_key_file_new();
-				if (!g_key_file_load_from_file(sub_kf, *p_str, 0, &gerr_local))
-					WARN("Configuration file [%s] not parsed : %s", *p_str,
+				GKeyFile *sub_kf = g_key_file_new();
+				if (!g_key_file_load_from_file(sub_kf, path, 0, &gerr_local))
+					WARN("Configuration file [%s] not parsed : %s", path,
 						gerr_local ? gerr_local->message : "");
 				else if (!_cfg_reload_file(sub_kf, TRUE, &gerr_local))
-					WARN("Configuration file [%s] not loaded : %s", *p_str,
+					WARN("Configuration file [%s] not loaded : %s", path,
 						gerr_local ? gerr_local->message : "");
 				else
-					INFO("Loaded service file [%s]", *p_str);
+					INFO("Loaded service file [%s]", path);
 
 				if (gerr_local)
 					g_clear_error(&gerr_local);
